
#include "BitmapSerializer.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>


// -----------------------------------------------------------------------
//  BMP builder

void CBuilderBMP::build( FILE* file, const CCanvas& canvas )
{
    BITMAPFILEHEADER fhead;
    BITMAPINFOHEADER ihead;

	// HACK code follows. We reject non 32 bpp images
	assert( !(canvas.getWidth()&3) || !"Width must be a multiple of 4!" );

	ihead.biSize = sizeof(BITMAPINFOHEADER);
	ihead.biWidth = canvas.getWidth();
	ihead.biHeight = canvas.getHeight();
	ihead.biPlanes = 1;
	ihead.biBitCount = 24; //bpp; // HACK - we save as 24 bpp
	ihead.biCompression = BI_RGB;
	ihead.biSizeImage = 0; // rgb
	ihead.biXPelsPerMeter = 72;
	ihead.biYPelsPerMeter = 72;
	ihead.biClrUsed = 0;
	ihead.biClrImportant = 0;

	fhead.bfType = 0x4d42; // hardcore (from MSDN): 0x42 = "B" 0x4d = "M" 
	fhead.bfSize = sizeof(BITMAPFILEHEADER) + ihead.biSize + (canvas.getWidth() * canvas.getHeight() * 3);
	fhead.bfReserved1 = 0;
	fhead.bfReserved2 = 0;
	fhead.bfOffBits = sizeof(BITMAPFILEHEADER) + ihead.biSize; 

	// write to file
	fwrite( &fhead, sizeof(fhead), 1, file );
	fwrite( &ihead, sizeof(ihead), 1, file );

	// save as 24 bpp, though...
	for( int y = canvas.getHeight()-1; y >= 0; y-- ) {
		const CCanvas::pixel *d = canvas.getRow( y );
		for( int x = 0; x < canvas.getWidth(); x++ ) {
			CCanvas::pixel pix = d[x];
			fwrite( &pix, 3, 1, file );
		}
	}
}


CCanvas* CBuilderBMP::reverseBuild( FILE* file )
{
    BITMAPFILEHEADER fhead;
    BITMAPINFOHEADER ihead;

	fread( &fhead, sizeof(fhead), 1, file );
	fread( &ihead, sizeof(ihead), 1, file );

	if( ihead.biWidth & 3 ) // kind of HACK
		return NULL;
	if( ihead.biBitCount != 24 )
		return NULL;
	if( ihead.biCompression != 0 )
		return NULL;

	CCanvas* cnv = new CCanvas( ihead.biWidth, ihead.biHeight );

	// load data
	for( int i = ihead.biHeight-1; i >= 0; i-- ) {
		CCanvas::pixel* p = cnv->getRow( i );
		for( int j = 0; j < ihead.biWidth; j++ ) {
			CCanvas::pixel pix = 0;
			fread( &pix, 3, 1, file );
			*p = pix;
			p++;
		}
	}

	return cnv;
}
