
#include "SpikeRule.h"
#include "../util/Random.h"


CRandom CSpikeModifier::mRand;


void CSpikeModifier::apply( const CSpike& spike, CPool<CSpike>* pools ) const
{
	SVector origin = spike.getOrigin() + spike.getDirection() * mOriginOffset;
	SVector dir = spike.getDirection() * mLenScale;
	if( mRotX != 0.0f || mRotY != 0.0f || mRotZ != 0.0f ) {
		if( mRandom ) {
			float rx = mRotX * (mRand.getFloat()-0.5f) * 2.0f;
			float ry = mRotY * (mRand.getFloat()-0.5f) * 2.0f;
			float rz = mRotZ * (mRand.getFloat()-0.5f) * 2.0f;
			dir.rotate( rx, ry, rz );
		} else
			dir.rotate( mRotX, mRotY, mRotZ );
	}
	
	pools[mCategory].add( CSpike( origin, dir ) );
}


void CSpikeRule::apply( const CSpike& spike, int vars, CPool<CSpike>* pools ) const
{
	// HACK-y speedup
	float len = spike.getDirection().squaredLength();
	if( len < 0.004f * 0.004f )
		return;

	TModifierVector::const_iterator it;
	for( it = mModifiers.begin(); it != mModifiers.end(); ++it )
		(*it).apply( spike, pools );
}

