
#ifndef __SPIKE_SYSTEM_RENDERER_H
#define __SPIKE_SYSTEM_RENDERER_H

#include "../render/Canvas.h"
#include "../render/Drawing.h"
#include "SpikeSystem.h"

#include <vector>
#include <algorithm>
#include <cmath>
#include <cstdio>


class CSpikeComparator {
public:
	// back to front
	bool operator() ( const CSpike*& a, const CSpike*& b ) const {
		return a->getOrigin().z > b->getOrigin().z;
	}
};



template< class SPKRENDER >
class CSpikeSystemRenderer {
public:
	typedef std::vector<const CSpike*>	TSpikeVector;
public:
	static void render( SPKRENDER& rnd, CDrawing& draw, CCanvas& color, CCanvas& depth, CSpikeSystem& system );
};


template< class SPKRENDER >
void CSpikeSystemRenderer<SPKRENDER>::render( SPKRENDER& rnd, CDrawing& draw, CCanvas& color, CCanvas& depth, CSpikeSystem& system )
{
	TSpikeVector spikes;
	spikes.reserve( 1000 );
	
	//
	// extract
	
	printf( "get " ); fflush( stdout );
	system.getSystem().getSpikes( spikes );
	printf( "(%i) ", spikes.size() );
	
	//
	// sort

	printf( "sort " ); fflush( stdout );
	std::sort( spikes.begin(), spikes.end(), CSpikeComparator() );

	//
	// bounding box

	printf( "bbox " ); fflush( stdout );

	float xmax, xmin, ymax, ymin, zmax, zmin;

	xmax = ymax = zmax = -1.0e6;
	xmin = ymin = zmin = 1.0e6;

	TSpikeVector::const_iterator cit;
	for( cit = spikes.begin(); cit != spikes.end(); cit++ ) {
		const CSpike& spk = **cit;
		SVector v = spk.getOrigin() + spk.getDirection();
		if( v.x > xmax ) xmax = v.x;
		if( v.y > ymax ) ymax = v.y;
		if( v.z > zmax ) zmax = v.z;
		if( v.x < xmin ) xmin = v.x;
		if( v.y < ymin ) ymin = v.y;
		if( v.z < zmin ) zmin = v.z;
		v = spk.getOrigin();
		if( v.x > xmax ) xmax = v.x;
		if( v.y > ymax ) ymax = v.y;
		if( v.z > zmax ) zmax = v.z;
		if( v.x < xmin ) xmin = v.x;
		if( v.y < ymin ) ymin = v.y;
		if( v.z < zmin ) zmin = v.z;
	}

	float xmid, ymid;
	xmid = (xmin + xmax) * 0.5f;
	ymid = (ymin + ymax) * 0.5f;

	float scale;
	float xscale = (float)(color.getWidth()) / (xmax-xmin+0.1f);
	float yscale = (float)(color.getHeight()) / (ymax-ymin+0.1f);
	scale = xscale < yscale ? xscale : yscale;
	scale *= 0.95f;

	float zrange = zmax-zmin+0.1f;

	printf( "draw " ); fflush( stdout );

	rnd.begin( draw, color, depth, zrange );

	for( cit = spikes.begin(); cit != spikes.end(); ++cit ) {
		const CSpike& sp = **cit;
		SVector v1 = sp.getOrigin();
		SVector v2 = sp.getOrigin() + sp.getDirection();

		v1.x -= xmid;
		v1.y -= ymid;
		v2.x -= xmid;
		v2.y -= ymid;

		v1.x *= scale;
		v1.y *= scale;
		v2.x *= scale;
		v2.y *= scale;

		v1.z -= zmin;
		v2.z -= zmin;

		rnd.renderSpike( sp, v1, v2, scale );
	}
}


#endif
