
#include "SimpleEffects.h"

#include <cmath>
#include <cstring>


// ------------------------------------------------------------------------


void CBlurEffect::perform( CDrawing& draw, CCanvas& color, CCanvas& depth )
{
	CCanvas::pixel*	pcolor = color.getRow(mSize) + mSize;

	int n = (color.getHeight()-mSize*2) * color.getWidth() - mSize*2;
	for( int i = 0; i < n; i++ ) {
		CCanvas::pixel c1 = CCanvas::rgbAvg( pcolor[-mSize], pcolor[mSize] );
		CCanvas::pixel c2 = CCanvas::rgbAvg( pcolor[-color.getWidth()*mSize], pcolor[color.getWidth()*mSize] );
		*pcolor = CCanvas::rgbAvg( c1, c2 );
		pcolor++;
	}
}


// ------------------------------------------------------------------------


void CBackgroundEffect::perform( CDrawing& draw, CCanvas& color, CCanvas& depth )
{
	int i, j, k;

	CCanvas::pixel colWhite = CCanvas::rgb( 255, 255, 255 );
	CCanvas::pixel colBlack = CCanvas::rgb( 0, 0, 0 );
	
	//
	// fill uniformly

	draw.fill( color, mDark ? colBlack : colWhite );
	draw.fillFP( depth, 1.0e6f );

	//
	// some tones

	for( j = 0; j < 16; j++ ) {

		unsigned int r = 255 - (mRandom.getInt() & 127);
		unsigned int g = 255 - (mRandom.getInt() & 127);
		unsigned int b = 255 - (mRandom.getInt() & 127);
		CCanvas::pixel colRand = CCanvas::rgb( r, g, b );

		float t = 0.0f;
		float dt = 1.0f / color.getWidth();
		float r1 = mRandom.getFloat() * 4.0f;
		float r2 = mRandom.getFloat() * 4.0f;
		for( i = 0; i < color.getWidth(); i++ ) {
			int y1 = (int)((cosf( t * r1 * sinf( t * r2 ) + r2 ) * 0.5f + 0.5f ) * color.getHeight());
			int y2 = y1 + (int)((cosf( t * r2 * sinf( t * r1 ) + r1 ) + 1.2f) * color.getHeight() * 0.1f);
			if( y1 < 0 )
				y1 = 0;
			if( y2 < 0 )
				y2 = 0;
			if( y1 >= color.getHeight() )
				y1 = color.getHeight()-1;
			if( y2 >= color.getHeight() )
				y2 = color.getHeight()-1;

			draw.lineBlend( color, i, y1, i, y2, colRand, 3 );

			t += dt;
		}
	}

	 
	//
	// draw gridlines
	
	i = color.getHeight()-1;
	j = 2;
	k = 8*2;
	while( i >= 0 ) {
		draw.lineBlend( color, 0, i, color.getWidth()-1, i, mDark ? colWhite : colBlack, k/2 );
		i -= j;
		j++;
		if( k > 2 )
			k--;
	}
	i = color.getWidth()-1;
	j = 2;
	k = 8*2;
	while( i >= 0 ) {
		draw.lineBlend( color, i, 0, i, color.getHeight()-1, mDark ? colWhite : colBlack, k/2 );
		i -= j;
		j++;
		if( k > 2 )
			k--;
	}
}



// ------------------------------------------------------------------------


void CBackgroundEffect2::perform( CDrawing& draw, CCanvas& color, CCanvas& depth )
{
	int i, j, k;
	
	CCanvas::pixel colWhite = CCanvas::rgb( 255, 255, 255 );
	CCanvas::pixel colBlack = CCanvas::rgb( 0, 0, 0 );
	
	//
	// fill uniformly
	
	draw.fill( color, mDark ? colBlack : colWhite );
	draw.fillFP( depth, 1.0e6f );
	
	//
	// some tones
	
	for( j = 0; j < 6; j++ ) {
		
		unsigned int r = (mRandom.getInt() & 127);
		unsigned int g = (mRandom.getInt() & 127);
		unsigned int b = (mRandom.getInt() & 127);
		CCanvas::pixel colRand = CCanvas::rgb( r, g, b );
		
		float t = 0.0f;
		float dt = 1.0f / color.getWidth();
		float r1 = mRandom.getFloat() * 8.0f;
		float r2 = mRandom.getFloat() * 8.0f;
		for( i = 0; i < color.getHeight(); i++ ) {
			int x1 = (int)((cosf( t * r1 * atanf( t * r2 ) + r2 ) * 0.5f + 0.5f ) * color.getWidth());
			int x2 = x1 + (int)((atanf( t * r2 * cosf( t * r1 ) + r1 ) + 1.2f) * color.getWidth() * 0.1f);
			if( x1 < 0 )
				x1 = 0;
			if( x2 < 0 )
				x2 = 0;
			if( x1 >= color.getWidth() )
				x1 = color.getWidth()-1;
			if( x2 >= color.getWidth() )
				x2 = color.getWidth()-1;
			
			draw.lineBlend( color, x1, i, x2, i, colRand, 4 );
			
			t += dt;
		}
	}
	
	
	//
	// draw gridlines
	
	i = color.getHeight()-1;
	j = 2;
	k = 8*2;
	while( i >= 0 ) {
		draw.lineBlend( color, 0, color.getHeight()*3/4-i*3/4, color.getWidth()-1, i, mDark ? colWhite : colBlack, k/2 );
		i -= j;
		j++;
		if( k > 3 )
			k--;
	}
	i = color.getWidth()-1;
	j = 2;
	k = 8*2;
	while( i >= 0 ) {
		draw.lineBlend( color, color.getWidth()/4+i/2, 0, i, color.getHeight()-1, mDark ? colWhite : colBlack, k/2 );
		i -= j;
		j++;
		if( k > 3 )
			k--;
	}
}


// ------------------------------------------------------------------------

const char* CTextEffect::mLetters = " abcdefghijklmnopqrstuvwxyz0123456789.,:?";
int* CTextEffect::mIndexByChar;


void CTextEffect::initialize()
{
	int last = strlen(mLetters)-1;
	mIndexByChar = new int[256];
	for( int c = 0; c < 256; c++ ) {
		char* pos = strchr( mLetters, c );
		if( !pos )
			mIndexByChar[c] = last;
		else {
			int idx = pos - mLetters;
			if( idx > last )
				idx = last;
			mIndexByChar[c] = idx;
		}

	}
}


CTextEffect::CTextEffect( const CCanvas& image, int size )
:	mImage(image),
	mSize(size),
	mText(),
	mX(0), mY(0)
{
}


void CTextEffect::perform( CDrawing& draw, CCanvas& color, CCanvas& depth )
{
	if( !&mImage ) // HACK
		return;

	int x = mX;
	int y = mY;

	for( int i = 0; i < mText.length(); i++ ) {
		int pos = mIndexByChar[ mText[i] ];
		if( !mSub )
			draw.blitBlendSub( color, mImage, x, y, pos * mSize, 0, mSize, mImage.getHeight() );
		else
			draw.blitBlendAdd( color, mImage, x, y, pos * mSize, 0, mSize, mImage.getHeight() );
		x += mSize;
	}
}

