
#ifndef __CANVAS_H
#define __CANVAS_H


/**
 *  A truecolor, 32 bit XRGB memory canvas.
 */
class CCanvas {
public:
	typedef unsigned int pixel;
	static pixel rgb( int r, int g, int b ) { return (pixel)(((r&255)<<16)|((g&255)<<8)|(b&255)); }
	static pixel rgbAdd( pixel a, pixel b );
	static pixel rgbSub( pixel a, pixel b );
	static pixel rgbAvg( pixel a, pixel b );

public:
	CCanvas( int width, int height );
	~CCanvas();

	int getWidth() const { return mWidth; }
	int getHeight() const { return mHeight; }
	
	const pixel* getData() const { return mData; }
	pixel* getData() { return mData; }

	const float* getFPData() const { return (const float*)mData; }
	float* getFPData() { return (float*)mData; }

	const pixel* getRow( int y ) const { return mRowPtrs[y]; }
	pixel* getRow( int y ) { return mRowPtrs[y]; }

	const float* getFPRow( int y ) const { return (const float*)mRowPtrs[y]; }
	float* getFPRow( int y ) { return (float*)mRowPtrs[y]; }

private:
	int		mWidth;
	int		mHeight;
	pixel*	mData;

	/** Individual row pointers for quick access on slow computers :) */
	pixel**	mRowPtrs;
};


inline CCanvas::pixel CCanvas::rgbAdd( pixel pa, pixel pb )
{
	unsigned int r = (pa & 0xFF0000) + (pb & 0xFF0000);
	unsigned int g = (pa & 0x00FF00) + (pb & 0x00FF00);
	unsigned int b = (pa & 0x0000FF) + (pb & 0x0000FF);
	if( r > 0xFF0000 ) r = 0xFF0000;
	if( g > 0x00FF00 ) g = 0x00FF00;
	if( b > 0x0000FF ) b = 0x0000FF;
	return r | g | b;
}

inline CCanvas::pixel CCanvas::rgbSub( pixel pa, pixel pb )
{
	int r = (pa & 0xFF0000) - (pb & 0xFF0000);
	int g = (pa & 0x00FF00) - (pb & 0x00FF00);
	int b = (pa & 0x0000FF) - (pb & 0x0000FF);
	if( r < 0 ) r = 0;
	if( g < 0 ) g = 0;
	if( b < 0 ) b = 0;
	return r | g | b;
}

inline CCanvas::pixel CCanvas::rgbAvg( pixel pa, pixel pb )
{
	unsigned int r = ((pa & 0xFF0000) + (pb & 0xFF0000)) >> 1;
	unsigned int g = ((pa & 0x00FF00) + (pb & 0x00FF00)) >> 1;
	unsigned int b = ((pa & 0x0000FF) + (pb & 0x0000FF)) >> 1;
	r &= 0xFF0000;
	g &= 0x00FF00;
	b &= 0x0000FF;
	if( r > 0xFF0000 ) r = 0xFF0000;
	if( g > 0x00FF00 ) g = 0x00FF00;
	if( b > 0x0000FF ) b = 0x0000FF;
	return r | g | b;
}


#endif
